/*****************************************************************************
 *
 * LOGS.C - Logs functions for Nagios
 *
 *  * Last Modified:   08-19-2002
 *
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************/

#include "config.h"
#include "common.h"
#include "logs.h"
#include "objects.h"

/***** IMPLEMENTATION-SPECIFIC INCLUDES *****/

/* #ifdef USE_XCDDEFAULT */
/* #include "../xdata/xcddefault.h" */
/* #endif */

#ifdef USE_XLDDB
#include "../xdata/xlddb.h"
#endif

#ifdef NSCORE
#include "../base/nagios.h"
#endif

/* #ifdef NSCGI */
/* #include "../cgi/cgiutils.h" */
/* #endif */

/* #ifdef NSCGI */
/* log     *log_list=NULL; */
/* #endif */




#ifdef NSCORE

/******************************************************************/
/**************** INITIALIZATION/CLEANUP FUNCTIONS ****************/
/******************************************************************/


/* initalizes log data */
int initialize_log_data(char *config_file){
	int result;

	/**** IMPLEMENTATION-SPECIFIC CALLS ****/
	/* #ifdef USE_XLDDEFAULT */
/* 	result=xlddefault_initialize_log_data(config_file); */
/* 	#endif */

#ifdef USE_XLDDB
	result=xlddb_initialize_log_data(config_file);
#endif
	return result;
        }


/* removes old/invalid comments */
int cleanup_log_data(char *config_file){
	int result;
	
	/**** IMPLEMENTATION-SPECIFIC CALLS ****/
	/* #ifdef USE_XCDDEFAULT */
/* 	result=xlddefault_cleanup_log_data(config_file); */
/* 	#endif */

#ifdef USE_XLDDB
	result=xlddb_cleanup_log_data(config_file);
#endif
	return result;
        }



/* /\******************************************************************\/ */
/* /\****************** LOG OUTPUT FUNCTIONS **********************\/ */
/* /\******************************************************************\/ */


/* saves a host or service log */
int save_log(int type, char *name, char *description, char *state, char *state_type, char *current_attempt, char *plugin_output){
 	int result;
	
 	if(type==HOST_LOG)
	  result=save_host_log(name, state, state_type, current_attempt, plugin_output);
 	else
	  result=save_service_log(name, description, state, state_type, current_attempt, plugin_output);
 	return result;
}

/* saves a host log */
int save_host_log(char *host_name, char *host_state, char *host_state_type, char *current_host_attempt, char *plugin_output){
 	int result;
	int dummy;
	time_t t;

	time(&t);

 	/**** IMPLEMENTATION-SPECIFIC CALLS ****/
/* #ifdef USE_XCDDEFAULT */
/* 	result=xcddefault_save_host_comment(host_name,entry_time,author_name,comment_data,persistent,comment_id); */
/* #endif */
#ifdef USE_XLDDB
	result=xlddb_save_host_log(t,host_name,host_state,host_state_type,current_host_attempt,plugin_output,&dummy);
#endif
	
	return result;
}


/* saves a service log */
int save_service_log(char *host_name, char* service_description, char *service_state, char *service_state_type, char *current_service_attempt, char *plugin_output){
  int result;
  int dummy;
  time_t t;

  time(&t);  
  /**** IMPLEMENTATION-SPECIFIC CALLS ****/
/* #ifdef USE_XLDDEFAULT */
/*   result=xcddefault_save_service_comment(host_name,svc_description,entry_time,author_name,comment_data,persistent,comment_id); */
/* #endif */


#ifdef USE_XLDDB
  result=xlddb_save_service_log(t,host_name,service_description,service_state,service_state_type,current_service_attempt,plugin_output,&dummy);
#endif
  
  return result;
}



/******************************************************************/
/***************** LOG DELETION FUNCTIONS *********************/
/******************************************************************/


/* deletes a host or service log */
int delete_log(int type, int log_id){
	int result;

	if(type==HOST_LOG)
		result=delete_host_log(log_id);
	else
		result=delete_service_log(log_id);

	return result;
        }


/* deletes a host log */
int delete_host_log(int log_id){
	int result;
	
	/**** IMPLEMENTATION-SPECIFIC CALLS ****/
/* #ifdef USE_XLDDEFAULT */
/* 	result=xlddefault_delete_host_log(log_id); */
/* #endif */
#ifdef USE_XLDDB
	result=xlddb_delete_host_log(log_id,FALSE);
#endif

	return result;
}



/* deletes a service log */
int delete_service_log(int log_id){
	int result;
	
	/**** IMPLEMENTATION-SPECIFIC CALLS ****/
/* #ifdef USE_XLDDEFAULT */
/* 	result=xlddefault_delete_service_log(log_id); */
/* #endif */
#ifdef USE_XLDDB
	result=xlddb_delete_service_log(log_id,FALSE);
#endif

	return result;
}


/* deletes all logs for a particular host or service */
int delete_all_logs(int type, char *host_name, char *svc_description){
	int result;

	if(type==HOST_LOG)
		result=delete_all_host_logs(host_name);
	else
		result=delete_all_service_logs(host_name,svc_description);

	return result;
        }


/* deletes all logs for a particular host */
int delete_all_host_logs(char *host_name){
	int result;

	/**** IMPLEMENTATION-SPECIFIC CALLS ****/
/* #ifdef USE_XLDDEFAULT */
/* 	result=xlddefault_delete_all_host_logs(host_name); */
/* #endif */
#ifdef USE_XLDDB
	result=xlddb_delete_all_host_logs(host_name);
#endif

	return result;
        }


/* deletes all logs for a particular service */
int delete_all_service_logs(char *host_name, char *svc_description){
	int result;

	/**** IMPLEMENTATION-SPECIFIC CALLS ****/
/* #ifdef USE_XLDDEFAULT */
/* 	result=xlddefault_delete_all_service_logs(host_name,svc_description); */
/* #endif */
#ifdef USE_XLDDB
	result=xlddb_delete_all_service_logs(host_name,svc_description);
#endif

	return result;
        }

#endif




/* #ifdef NSCGI */

/* /\******************************************************************\/ */
/* /\********************** INPUT FUNCTIONS ***************************\/ */
/* /\******************************************************************\/ */


/* int read_log_data(char *main_config_file){ */
/* 	int result; */

/* 	/\**** IMPLEMENTATION-SPECIFIC CALLS ****\/ */
/* #ifdef USE_XLDDEFAULT */
/* 	result=xlddefault_read_log_data(main_config_file); */
/* #endif */
/* #ifdef USE_XLDDB */
/* 	result=xlddb_read_log_data(main_config_file); */
/* #endif */

/* 	return result; */
/*         } */



/* /\******************************************************************\/ */
/* /\******************** ADDITION FUNCTIONS **************************\/ */
/* /\******************************************************************\/ */


/* /\* adds a host log to the list in memory *\/ */
/* int add_host_log(char *host_name, time_t entry_time, char *author, char *log_data, int log_id, int persistent){ */
/* 	int result; */

/* 	result=add_log(HOST_LOG,host_name,NULL,entry_time,author,log_data,log_id,persistent); */

/* 	return result; */
/*         } */



/* /\* adds a service log to the list in memory *\/ */
/* int add_service_log(char *host_name, char *svc_description, time_t entry_time, char *author, char *log_data, int log_id, int persistent){ */
/* 	int result; */

/* 	result=add_log(SERVICE_LOG,host_name,svc_description,entry_time,author,log_data,log_id,persistent); */

/* 	return result; */
/*         } */



/* /\* adds a log to the list in memory *\/ */
/* int add_log(int log_type, char *host_name, char *svc_description, time_t entry_time, char *author, char *log_data, int log_id, int persistent){ */
/* 	log *new_log=NULL; */
/* 	log *last_log=NULL; */
/* 	log *temp_log=NULL; */

/* 	/\* allocate memory for the log *\/ */
/* 	new_log=(log *)malloc(sizeof(log)); */
/* 	if(new_log==NULL) */
/* 		return ERROR; */

/* 	new_log->host_name=strdup(host_name); */
/* 	if(new_log->host_name==NULL){ */
/* 		free(new_log); */
/* 		return ERROR; */
/* 	        } */

/* 	if(log_type==SERVICE_LOG){ */
/* 		new_log->service_description=strdup(svc_description); */
/* 		if(new_log->service_description==NULL){ */
/* 			free(new_log->host_name); */
/* 			free(new_log); */
/* 			return ERROR; */
/* 		        } */
/* 	        } */
/* 	else */
/* 		new_log->service_description=NULL; */

/* 	new_log->author=strdup(author); */
/* 	if(new_log->author==NULL){ */
/* 		if(new_log->service_description!=NULL) */
/* 			free(new_log->service_description); */
/* 		free(new_log->host_name); */
/* 		free(new_log); */
/* 		return ERROR; */
/* 	        } */

/* 	new_log->log_data=strdup(log_data); */
/* 	if(new_log->log_data==NULL){ */
/* 		free(new_log->author); */
/* 		if(new_log->service_description!=NULL) */
/* 			free(new_log->service_description); */
/* 		free(new_log->host_name); */
/* 		free(new_log); */
/* 		return ERROR; */
/* 	        } */

/* 	new_log->log_type=log_type; */
/* 	new_log->entry_time=entry_time; */
/* 	new_log->log_id=log_id; */
/* 	new_log->persistent=(persistent>0)?TRUE:FALSE; */


/* 	/\* add new log to log list, sorted by log id *\/ */
/* 	last_log=log_list; */
/* 	for(temp_log=log_list;temp_log!=NULL;temp_log=temp_log->next){ */
/* 		if(new_log->log_id<temp_log->log_id){ */
/* 			new_log->next=temp_log; */
/* 			if(temp_log==log_list) */
/* 				log_list=new_log; */
/* 			else */
/* 				last_log->next=new_log; */
/* 			break; */
/* 		        } */
/* 		else */
/* 			last_log=temp_log; */
/* 	        } */
/* 	if(log_list==NULL){ */
/* 		new_log->next=NULL; */
/* 		log_list=new_log; */
/* 	        } */
/* 	else if(temp_log==NULL){ */
/* 		new_log->next=NULL; */
/* 		last_log->next=new_log; */
/* 	        } */

/* 	return OK; */
/*         } */




/* /\******************************************************************\/ */
/* /\********************* CLEANUP FUNCTIONS **************************\/ */
/* /\******************************************************************\/ */

/* /\* frees memory allocated for the log data *\/ */
/* void free_log_data(void){ */
/* 	log *this_log; */
/* 	log *next_log; */

/* 	/\* free memory for the log list *\/ */
/* 	for(this_log=log_list;this_log!=NULL;this_log=next_log){ */
/* 		next_log=this_log->next; */
/* 		free(this_log->host_name); */
/* 		free(this_log->service_description); */
/* 		free(this_log->author); */
/* 		free(this_log->log_data); */
/* 		free(this_log); */
/* 	        } */

/* 	/\* reset list pointer *\/ */
/* 	log_list=NULL; */

/* 	return; */
/*         } */



/* /\******************************************************************\/ */
/* /\********************* UTILITY FUNCTIONS **************************\/ */
/* /\******************************************************************\/ */

/* /\* get the number of logs associated wth a particular host *\/ */
/* int number_of_host_logs(char *host_name){ */
/* 	log *temp_log; */
/* 	int total_logs=0; */

/* 	if(host_name==NULL) */
/* 		return 0; */

/* 	for(temp_log=log_list;temp_log!=NULL;temp_log=temp_log->next){ */
/* 		if(temp_log->log_type==HOST_LOG && !strcmp(temp_log->host_name,host_name)) */
/* 			total_logs++; */
/* 	        } */

/* 	return total_logs; */
/*         } */


/* /\* get the number of logs associated wth a particular service *\/ */
/* int number_of_service_logs(char *host_name, char *svc_description){ */
/* 	log *temp_log; */
/* 	int total_logs=0; */

/* 	if(host_name==NULL || svc_description==NULL) */
/* 		return 0; */

/* 	for(temp_log=log_list;temp_log!=NULL;temp_log=temp_log->next){ */
/* 		if(temp_log->log_type==SERVICE_LOG && !strcmp(temp_log->host_name,host_name) && !strcmp(temp_log->service_description,svc_description)) */
/* 			total_logs++; */
/* 	        } */

/* 	return total_logs; */
/*         } */


/* /\* find a service log by id *\/ */
/* log *find_service_log(int log_id){ */
	
/* 	return find_log(log_id,SERVICE_LOG); */
/*         } */


/* /\* find a host log by id *\/ */
/* log *find_host_log(int log_id){ */
	
/* 	return find_log(log_id,HOST_LOG); */
/*         } */


/* /\* find a log by id *\/ */
/* log *find_log(int log_id, int log_type){ */
/* 	log *temp_log; */

/* 	for(temp_log=log_list;temp_log!=NULL;temp_log=temp_log->next){ */
/* 		if(temp_log->log_id==log_id && temp_log->log_type==log_type) */
/* 			return temp_log; */
/* 	        } */

/* 	return NULL; */
/*         } */


/* #endif */



