/*****************************************************************************
 *
 * XLDDB.C - Database routines for log data
 *
 * Copyright (c) 2000-2002 Ethan Galstad (nagios@nagios.org)
 * Last Modified:   08-04-2004 by Christophe Paris
 *
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************/


/*********** COMMON HEADER FILES ***********/

#include "../common/config.h"
#include "../common/common.h"
#include "../common/locations.h"

#ifdef NSCORE
#include "../common/objects.h"
#include "../base/nagios.h"
#endif

#ifdef NSCGI
#include "../cgi/cgiutils.h"
#endif


/**** IMPLEMENTATION SPECIFIC HEADER FILES ****/

#include "xlddb.h"

#ifdef USE_XLDMYSQL
#include <mysql/mysql.h>
#include <mysql/errmsg.h>
#endif

#ifdef USE_XLDPGSQL
#include <pgsql/libpq-fe.h>
#endif


char		xlddb_host[XLDDB_BUFFER_LENGTH]="";
char		xlddb_username[XLDDB_BUFFER_LENGTH]="";
char		xlddb_password[XLDDB_BUFFER_LENGTH]="";
char		xlddb_database[XLDDB_BUFFER_LENGTH]="";
int		xlddb_port=3306;
int		xlddb_optimize_data=TRUE;

#ifdef USE_XLDMYSQL
MYSQL		xlddb_mysql;
#endif

#ifdef USE_XLDPGSQL
PGconn		*xlddb_pgconn=NULL;
PGresult	*xlddb_pgres=NULL;
#endif



/******************************************************************/
/***************** COMMON CONFIG INITIALIZATION  ******************/
/******************************************************************/

/* grab configuration information from appropriate config file(s) */
int xlddb_grab_config_info(char *config_file){
	char input_buffer[MAX_INPUT_BUFFER];
	FILE *fp;
#ifdef NSCORE
	FILE *fp2;
	char *temp_buffer;
#endif

	/* DBMS-specifc defaults */
#ifdef USE_XLDMYSQL
	xlddb_port=3306;
#endif
#ifdef USE_XLDPGSQL
	xlddb_port=5432;
#endif 

	/*** CORE PASSES IN MAIN CONFIG FILE, CGIS PASS IN CGI CONFIG FILE! ***/

	/* open the config file for reading */
	fp=fopen(config_file,"r");
	if(fp==NULL)
		return ERROR;

	/* read in all lines from the config file */
	for(fgets(input_buffer,sizeof(input_buffer)-1,fp);!feof(fp);fgets(input_buffer,sizeof(input_buffer)-1,fp)){

		/* skip blank lines and comments */
		if(input_buffer[0]=='#' || input_buffer[0]=='\x0' || input_buffer[0]=='\n' || input_buffer[0]=='\r')
			continue;

		strip(input_buffer);

#ifdef NSCORE
		/* core programs reads comment vars from resource config file */
		if(strstr(input_buffer,"resource_file")==input_buffer){

			temp_buffer=strtok(input_buffer,"=");
			temp_buffer=strtok(NULL,"\n");
			if(temp_buffer==NULL)
				continue;
			
			fp2=fopen(temp_buffer,"r");
			if(fp2==NULL)
				continue;

			/* read in all lines from the resource config file */
			for(fgets(input_buffer,sizeof(input_buffer)-1,fp2);!feof(fp2);fgets(input_buffer,sizeof(input_buffer)-1,fp2)){

				/* skip blank lines and comments */
				if(input_buffer[0]=='#' || input_buffer[0]=='\x0' || input_buffer[0]=='\n' || input_buffer[0]=='\r')
					continue;

				strip(input_buffer);

				xlddb_grab_config_directives(input_buffer); //ok
			        }

			fclose(fp2);
		        }
#endif

/* #ifdef NSCGI */
/* 		/\* CGIs read variables directly from the CGI config file *\/ */
/* 		xlddb_grab_config_directives(input_buffer); //ok */
/* #endif */
	        }

	fclose(fp);

	return OK;
        }


void xlddb_grab_config_directives(char *input_buffer){
	char *temp_buffer;

	/* server name/address */
	if(strstr(input_buffer,"xlddb_host")==input_buffer){
		temp_buffer=strtok(input_buffer,"=");
		temp_buffer=strtok(NULL,"\n");
		if(temp_buffer==NULL)
			return;
		strncpy(xlddb_host,temp_buffer,sizeof(xlddb_host)-1);
		xlddb_host[sizeof(xlddb_host)-1]='\x0';
	        }

	/* username */
	else if(strstr(input_buffer,"xlddb_username")==input_buffer){
		temp_buffer=strtok(input_buffer,"=");
		temp_buffer=strtok(NULL,"\n");
		if(temp_buffer==NULL)
			return;
		strncpy(xlddb_username,temp_buffer,sizeof(xlddb_username)-1);
		xlddb_username[sizeof(xlddb_username)-1]='\x0';
	        }

	/* password */
	else if(strstr(input_buffer,"xlddb_password")==input_buffer){
		temp_buffer=strtok(input_buffer,"=");
		temp_buffer=strtok(NULL,"\n");
		if(temp_buffer==NULL)
			return;
		strncpy(xlddb_password,temp_buffer,sizeof(xlddb_password)-1);
		xlddb_password[sizeof(xlddb_password)-1]='\x0';
	        }

	/* database */
	else if(strstr(input_buffer,"xlddb_database")==input_buffer){
		temp_buffer=strtok(input_buffer,"=");
		temp_buffer=strtok(NULL,"\n");
		if(temp_buffer==NULL)
			return;
		strncpy(xlddb_database,temp_buffer,sizeof(xlddb_database)-1);
		xlddb_database[sizeof(xlddb_database)-1]='\x0';
	        }

	/* port */
	else if(strstr(input_buffer,"xlddb_port")==input_buffer){
		temp_buffer=strtok(input_buffer,"=");
		temp_buffer=strtok(NULL,"\n");
		if(temp_buffer==NULL)
			return;
		xlddb_port=atoi(temp_buffer);
	        }

	/* optimize data option */
	else if(strstr(input_buffer,"xlddb_optimize_data")==input_buffer){
		temp_buffer=strtok(input_buffer,"=");
		temp_buffer=strtok(NULL,"\n");
		if(temp_buffer==NULL)
			return;
		xlddb_optimize_data=(atoi(temp_buffer)>0)?TRUE:FALSE;
	        }

	return;	
        }




/******************************************************************/
/******************** CONNECTION FUNCTIONS ************************/
/******************************************************************/


/* initialize database structures */
int xlddb_initialize(void){
#ifdef NSCORE
	char buffer[MAX_INPUT_BUFFER];
#endif

#ifdef USE_XLDMYSQL
	if(!mysql_init(&xlddb_mysql)){
#ifdef NSCORE
		snprintf(buffer,sizeof(buffer)-1,"Error: mysql_init() failed for log data\n");
		buffer[sizeof(buffer)-1]='\x0';
		write_to_logs_and_console(buffer,NSLOG_RUNTIME_ERROR,TRUE);
#endif
		return ERROR;
	        }
#endif

	return OK;
        }


/* try and connect to the database server */
int xlddb_connect(void){
#ifdef USE_XLDPGSQL
	char connect_string[XLDDB_BUFFER_LENGTH];
#endif
#ifdef NSCORE
	char buffer[MAX_INPUT_BUFFER];
#endif

#ifdef USE_XLDMYSQL
	/* establish a connection to the MySQL server */
	if(!mysql_real_connect(&xlddb_mysql,xlddb_host,xlddb_username,xlddb_password,xlddb_database,xlddb_port,NULL,0)){

		mysql_close(&xlddb_mysql);

#ifdef NSCORE
		snprintf(buffer,sizeof(buffer)-1,"Error: Could not connect to MySQL database '%s' on host '%s' using username '%s' and password '%s' for log data!\n",xlddb_database,xlddb_host,xlddb_username,xlddb_password);
		buffer[sizeof(buffer)-1]='\x0';
		write_to_logs_and_console(buffer,NSLOG_RUNTIME_ERROR,TRUE);
#endif

		return ERROR;
	        }
#endif


#ifdef USE_XLDPGSQL

	/* establish a connection to the PostgreSQL server */
	snprintf(connect_string,sizeof(connect_string)-1,"host='%s' port=%d dbname='%s' user='%s' password='%s'",xlddb_host,xlddb_port,xlddb_database,xlddb_username,xlddb_password);
	connect_string[sizeof(connect_string)-1]='\x0';
	xlddb_pgconn=PQconnectdb(connect_string);

	if(PQstatus(xlddb_pgconn)==CONNECTION_BAD){

		PQfinish(xlddb_pgconn);

#ifdef NSCORE
		snprintf(buffer,sizeof(buffer)-1,"Error: Could not connect to PostgreSQL database '%s' on host '%s' using username '%s' and password '%s' for log data!\n",xlddb_database,xlddb_host,xlddb_username,xlddb_password);
		buffer[sizeof(buffer)-1]='\x0';
		write_to_logs_and_console(buffer,NSLOG_RUNTIME_ERROR,TRUE);
#endif

		return ERROR;
	        }
#endif

	return OK;
        }


/* disconnect from database */
int xlddb_disconnect(void){

#ifdef USE_XLDMYSQL
	/* close the connection to the database server */		
	mysql_close(&xlddb_mysql);
#endif

#ifdef USE_XLDPGSQL
	/* close database connection and cleanup */
	if(PQstatus(xlddb_pgconn)!=CONNECTION_BAD)
		PQfinish(xlddb_pgconn);
#endif

	return OK;
        }


/* execute a SQL query */
int xlddb_query(char *sql_statement){
	int result=OK;

#ifdef DEBUG0
	printf("xlddb_query() start\n");
#endif

#ifdef DEBUG5
	printf("XLDDB_QUERY: %s\n",sql_statement);
#endif

#ifdef USE_XLDMYSQL
	if(mysql_query(&xlddb_mysql,sql_statement))
		result=ERROR;
#endif

#ifdef USE_XLDPGSQL
	xlddb_pgres=PQexec(xlddb_pgconn,sql_statement);
	if(xlddb_pgres==NULL || PQresultStatus(xlddb_pgres)==PGRES_FATAL_ERROR || PQresultStatus(xlddb_pgres)==PGRES_BAD_RESPONSE){
		PQclear(xlddb_pgres);
		result=ERROR;
	        }
#endif

#ifdef DEBUG5
	if(result==ERROR)
		printf("\tAn error occurred in the query!\n");
	printf("\tXLDDB_QUERY Done.\n");
#endif

#ifdef DEBUG0
	printf("xlddb_query() end\n");
#endif

	return result;
        }


/* frees memory associated with query results */
int xlddb_free_query_memory(void){

#ifdef DEBUG0
	printf("xlddb_free_query_memory() start\n");
#endif

#ifdef USE_XLDPGSQL
	PQclear(xlddb_pgres);
#endif
	
#ifdef DEBUG0
	printf("xlddb_free_query_memory() end\n");
#endif
	
	return OK;
        }



#ifdef NSCORE

/******************************************************************/
/************ LOG INITIALIZATION/CLEANUP FUNCTIONS ************/
/******************************************************************/


/* initialize log data */
int xlddb_initialize_log_data(char *main_config_file){
	char buffer[MAX_INPUT_BUFFER];

	/* grab configuration information */

	if(xlddb_grab_config_info(main_config_file)==ERROR){

		snprintf(buffer,sizeof(buffer)-1,"Error: Failed to grab database configuration information for log data\n");
		buffer[sizeof(buffer)-1]='\x0';
		write_to_logs_and_console(buffer,NSLOG_RUNTIME_ERROR,TRUE);
	
		return ERROR;
	        }

	/* initialize database structures */
	xlddb_initialize();

	/* connect to the db */
	if(xlddb_connect()==ERROR)
	    return ERROR;

	/* clean up the old log data */
	/* 	xlddb_validate_log_data(); */   /*no clean old logs*/

	/* optimize log tables */
	xlddb_optimize_tables();

	/* disconnect from db */
	xlddb_disconnect();

	return OK;
        }



/* removes invalid and old logs */  /*no clean old logs*/
int xlddb_validate_log_data(void){

	/* delete logs for hosts that don't exist anymore... */
	//xlddb_validate_host_logs();

	/* delete logs for services that don't exist anymore... */
	//xlddb_validate_service_logs();

	return OK;
        }



/* deletes logs for hosts that do not exist */
int xlddb_validate_host_logs(void){
	char sql_statement[XLDDB_SQL_LENGTH];
	host *temp_host=NULL;
#ifdef USE_XLDMYSQL
	MYSQL_RES *query_result=NULL;
	MYSQL_ROW result_row;
#endif
#ifdef USE_XLDPGSQL
	int ntuples;
	int tuple;
	int *deletes=NULL;
	int total_deletes=0;
	int current_delete=0;
#endif

#ifdef DEBUG0
	printf("xlddb_validate_host_logs() start\n");
#endif

	/* construct the SQL statement */
	snprintf(sql_statement,sizeof(sql_statement)-1,"SELECT hostlog_id, host_name FROM %s",XLDDB_HOSTLOGS_TABLE);
	sql_statement[sizeof(sql_statement)-1]='\x0';

	/* run the query... */
	if(xlddb_query(sql_statement)==ERROR){
		xlddb_free_query_memory();
		return ERROR;
	        }


#ifdef USE_XLDMYSQL
	/* get the results of the query */
	if((query_result=mysql_store_result(&xlddb_mysql))==NULL)
		return ERROR;

	/* fetch all rows... */
	while((result_row=mysql_fetch_row(query_result))!=NULL){

		/* find the host that this log is associated with */
		temp_host=find_host(result_row[1],NULL);

		/* if we couldn't find the host, delete this log */
		if(temp_host==NULL)
		  xlddb_delete_host_log(atoi(result_row[0]),TRUE); //!!!
                }
#endif

#ifdef USE_XLDPGSQL

	/* how many rows do we have? */
	ntuples=PQntuples(xlddb_pgres);

	/* allocate memory for storing IDs we're going to delete */
	deletes=(int *)malloc((ntuples+1)*sizeof(int));
	if(deletes==NULL){
		PQclear(xlddb_pgres);
		return ERROR;
	        }

	/* for each tuple... */
	for(tuple=0;tuple<ntuples;tuple++){

		/* find the host that this log is associated with */
		temp_host=find_host(PQgetvalue(xlddb_pgres,tuple,1),NULL);

		/* if we couldn't find the host, mark this log for deletion */
		if(temp_host==NULL)
			deletes[total_deletes++]=atoi(PQgetvalue(xlddb_pgres,tuple,0));
	        }

	/* free memory used to store query results */
	PQclear(xlddb_pgres);

	/* delete all records we should */
	for(current_delete=0;current_delete<total_deletes;current_delete++)
	  xlddb_delete_host_log(deletes[current_delete],TRUE); //!!
	
	/* free memory */
	free(deletes);
#endif

#ifdef DEBUG0
	printf("xlddb_validate_host_logs() end\n");
#endif

	return OK;
        }


/* deletes logs for service that do not exist */
int xlddb_validate_service_logs(void){
	char sql_statement[XLDDB_SQL_LENGTH];
	service *temp_service=NULL;
#ifdef USE_XLDMYSQL
	MYSQL_RES *query_result=NULL;
	MYSQL_ROW result_row;
#endif
#ifdef USE_XLDPGSQL
	int ntuples;
	int tuple;
	int *deletes=NULL;
	int current_delete=0;
	int total_deletes=0;
#endif

#ifdef DEBUG0
	printf("xlddb_validate_service_logs() start\n");
#endif

	/* construct the SQL statement */
	snprintf(sql_statement,sizeof(sql_statement)-1,"SELECT servicelog_id, host_name, service_description FROM %s",XLDDB_SERVICELOGS_TABLE);
	sql_statement[sizeof(sql_statement)-1]='\x0';

	/* run the query... */
	if(xlddb_query(sql_statement)==ERROR){
		xlddb_free_query_memory();
		return ERROR;
	        }

#ifdef USE_XLDMYSQL
	/* get the results of the query */
	if((query_result=mysql_store_result(&xlddb_mysql))==NULL)
		return ERROR;

	/* fetch all rows... */
	while((result_row=mysql_fetch_row(query_result))!=NULL){

		/* find the service that this log is associated with */
		temp_service=find_service(result_row[1],result_row[2],NULL);

		/* if we couldn't find the service, delete this log */
		if(temp_service==NULL)
			xlddb_delete_service_log(atoi(result_row[0]),TRUE);
                }
#endif

#ifdef USE_XLDPGSQL

	/* how many rows do we have? */
	ntuples=PQntuples(xlddb_pgres);

	/* allocate memory for storing IDs we're going to delete */
	deletes=(int *)malloc((ntuples+1)*sizeof(int));
	if(deletes==NULL){
		PQclear(xlddb_pgres);
		return ERROR;
	        }

	/* for each tuple... */
	for(tuple=0;tuple<ntuples;tuple++){

		/* find the service that this log is associated with */
		temp_service=find_service(PQgetvalue(xlddb_pgres,tuple,1),PQgetvalue(xlddb_pgres,tuple,2),NULL);

		/* if we couldn't find the service, mark this log for deletion */
		if(temp_service==NULL)
			deletes[total_deletes++]=atoi(PQgetvalue(xlddb_pgres,tuple,0));
	        }

	/* free memory used to store query results */
	PQclear(xlddb_pgres);

	/* delete all records we should */
	for(current_delete=0;current_delete<total_deletes;current_delete++)
		xlddb_delete_service_log(deletes[current_delete],TRUE);
	
	/* free memory */
	free(deletes);
#endif

#ifdef DEBUG0
	printf("xlddb_validate_service_logs() end\n");
#endif

	return OK;
        }



/* deinit log data */
int xlddb_cleanup_log_data(char *main_config_file){

	/* we don't have to do anything... */

	return OK;
        }




/**********************************************************/
/**************** OPTIMIZATION FUNCTIONS ******************/
/**********************************************************/

/* optimizes data in log tables */
int xlddb_optimize_tables(void){
#ifdef USE_XLDPGSQL
	char sql_statement[XLDDB_SQL_LENGTH];
#endif

#ifdef DEBUG0
	printf("xlddb_optimize_tables() start\n");
#endif

	/* don't optimize tables if user doesn't want us to */
	if(xlddb_optimize_data==FALSE)
		return OK;

#ifdef USE_XLDPGSQL
	
	/* VACUUM host and service log tables (user we're authenticated as must be owner of tables) */


	snprintf(sql_statement,sizeof(sql_statement)-1,"VACUUM %s",XLDDB_HOSTLOGS_TABLE);
	sql_statement[sizeof(sql_statement)-1]='\x0';
	xlddb_query(sql_statement);
	xlddb_free_query_memory();

	snprintf(sql_statement,sizeof(sql_statement)-1,"VACUUM %s",XLDDB_SERVICELOGS_TABLE);
	sql_statement[sizeof(sql_statement)-1]='\x0';
	xlddb_query(sql_statement);
	xlddb_free_query_memory();

#endif

#ifdef DEBUG0
	printf("xlddb_optimize_tables() end\n");
#endif

	return OK;
        }




/******************************************************************/
/***************** DEFAULT DATA OUTPUT FUNCTIONS ******************/
/******************************************************************/


/* saves a new host log */
int xlddb_save_host_log(time_t entry_time,char *host_name, char *host_state, char *host_state_type, char *current_host_attempt, char *plugin_output, int *log_id){
	char sql_statement[XLDDB_SQL_LENGTH];
	char buffer[MAX_INPUT_BUFFER];
	int result;
	char *escaped_host_name;
	char *escaped_host_state;
	char *escaped_host_state_type;
	char *escaped_current_host_attempt; 
	char *escaped_plugin_output;

#ifdef DEBUG0
	printf("xlddb_save_host_log() start\n");
#endif

	/* connect to the database */
	if(xlddb_connect()==ERROR)
	    return ERROR;
	/* escape the strings, as they may have quotes, etc... */
	escaped_host_name =(char *)malloc(strlen(host_name)*2+1);
	xlddb_escape_string(escaped_host_name,host_name);
	escaped_host_state =(char *)malloc(strlen(host_state)*2+1);
	xlddb_escape_string(escaped_host_state,host_state);
	escaped_host_state_type =(char *)malloc(strlen(host_state_type)*2+1);
	xlddb_escape_string(escaped_host_state_type,host_state_type);
	escaped_current_host_attempt = (char *)malloc(strlen(current_host_attempt)*2+1);
	xlddb_escape_string(escaped_current_host_attempt,current_host_attempt);
	escaped_plugin_output =(char *)malloc(strlen(plugin_output)*2+1);
	xlddb_escape_string(escaped_plugin_output,plugin_output);
	/* construct the SQL statement */
#ifdef USE_XLDMYSQL
	snprintf(sql_statement,sizeof(sql_statement)-1,"INSERT INTO %s (entry_time,host_name,host_state,host_state_type,current_host_attempt,plugin_output) VALUES (FROM_UNIXTIME(%lu),'%s','%s','%s','%s','%s')",XLDDB_HOSTLOGS_TABLE,entry_time,escaped_host_name,escaped_host_state,escaped_host_state_type,escaped_current_host_attempt,escaped_plugin_output);
#endif
#ifdef USE_XLDPGSQL
	snprintf(sql_statement,sizeof(sql_statement)-1,"INSERT INTO %s (entry_time,host_name,host_state,host_state_type,current_host_attempt,plugin_output) VALUES (%lu::abstime::timestamp,'%s','%s','%s','%s','%s')",XLDDB_HOSTLOGS_TABLE,entry_time,escaped_host_name,escaped_host_state,escaped_host_state_type,escaped_current_host_attempt,escaped_plugin_output);
#endif
	sql_statement[sizeof(sql_statement)-1]='\x0';
	/* free memory for the escaped strings */
	free(escaped_host_name);
	free(escaped_host_state);
	free(escaped_host_state_type);
	free(escaped_current_host_attempt);
	free(escaped_plugin_output);
	/* add the host log */
	result=xlddb_query(sql_statement);

	/* there was an error adding the log... */
	if(result==ERROR){
	  
	  xlddb_free_query_memory();
	  
	  snprintf(buffer,sizeof(buffer)-1,"Error: Could not insert record for host log in table '%s' of database '%s'\n",XLDDB_HOSTLOGS_TABLE,xlddb_database);
	  buffer[sizeof(buffer)-1]='\x0';
	  write_to_logs_and_console(buffer,NSLOG_RUNTIME_ERROR,TRUE);
	  
	  /* return an invalid log number */
	  if(log_id!=NULL)
	    *log_id=-1;
	  
	  /* disconnect from the database */
	  xlddb_disconnect();
	  return ERROR;
	}
	
	/* pass back the id of the log we just added */
	if(log_id!=NULL){
#ifdef USE_XLDMYSQL
	  *log_id=(int)mysql_insert_id(&xlddb_mysql);
#endif
#ifdef USE_XLDPGSQL
	  snprintf(sql_statement,sizeof(sql_statement)-1,"SELECT hostlog_id FROM %s WHERE oid='%s'",XLDDB_HOSTLOGS_TABLE,PQoidStatus(xlddb_pgres));
	  sql_statement[sizeof(sql_statement)-1]='\x0';
	  xlddb_free_query_memory();
	  result=xlddb_query(sql_statement);
	  if(result==ERROR)
	    *log_id=-1;
	  else{
	    if(PQntuples(xlddb_pgres)<1)
	      *log_id=-1;
	    else
	      *log_id=atoi(PQgetvalue(xlddb_pgres,0,0));
	  }
#endif
	}

	xlddb_free_query_memory();

	/* disconnect from the database */
	xlddb_disconnect();
#ifdef DEBUG0
	printf("xlddb_save_host_log() end\n");
#endif

	return OK;
        }


/* saves a new service log */
int xlddb_save_service_log(time_t entry_time, char *host_name, char *svc_description, char *svc_state, char *svc_state_type, char *current_svc_attempt, char *plugin_output,  int *log_id){
	char sql_statement[XLDDB_SQL_LENGTH];
	char buffer[MAX_INPUT_BUFFER];
	int result;
	char *escaped_host_name;
	char *escaped_svc_description;
	char *escaped_svc_state;
	char *escaped_svc_state_type;
	char *escaped_current_svc_attempt; 
	char *escaped_plugin_output;

#ifdef DEBUG0
	printf("xlddb_save_service_log() start\n");
#endif

	/* connect to the database */
	if(xlddb_connect()==ERROR)
		return ERROR;

	/* escape the strings, as they may have quotes, etc... */
	escaped_host_name=(char *)malloc(strlen(host_name)*2+1);
	xlddb_escape_string(escaped_host_name,host_name);
	escaped_svc_description=(char *)malloc(strlen(svc_description)*2+1);
	xlddb_escape_string(escaped_svc_description,svc_description);
	escaped_svc_state=(char *)malloc(strlen(svc_state)*2+1);
	xlddb_escape_string(escaped_svc_state,svc_state);
	escaped_svc_state_type=(char *)malloc(strlen(svc_state_type)*2+1);
	xlddb_escape_string(escaped_svc_state_type,svc_state_type);
	escaped_current_svc_attempt=(char *)malloc(strlen(current_svc_attempt)*2+1); 
	xlddb_escape_string(escaped_current_svc_attempt,current_svc_attempt);
	escaped_plugin_output=(char *)malloc(strlen(plugin_output)*2+1);
	xlddb_escape_string(escaped_plugin_output,plugin_output);


	/* construct the SQL statement */
#ifdef USE_XLDMYSQL
	snprintf(sql_statement,sizeof(sql_statement)-1,"INSERT INTO %s (entry_time,host_name,service_description,service_state,service_state_type,current_service_attempt,plugin_output) VALUES (FROM_UNIXTIME(%lu),'%s','%s','%s','%s','%s','%s')",XLDDB_SERVICELOGS_TABLE,entry_time,escaped_host_name,escaped_svc_description,escaped_svc_state,escaped_svc_state_type,escaped_current_svc_attempt,escaped_plugin_output);
#endif
#ifdef USE_XLDPGSQL
	snprintf(sql_statement,sizeof(sql_statement)-1,"INSERT INTO %s (entry_time,host_name,service_description,service_state,service_state_type,current_service_attempt,plugin_output) VALUES (%lu::abstime::timestamp,'%s','%s','%s','%s','%s','%s')",XLDDB_SERVICELOGS_TABLE,entry_time,escaped_host_name,escaped_svc_description,escaped_svc_state,escaped_svc_state_type,escaped_current_svc_attempt,escaped_plugin_output);
#endif
	sql_statement[sizeof(sql_statement)-1]='\x0';

	/* free memory for the escaped strings */
	free(escaped_host_name);
	free(escaped_svc_description);
	free(escaped_svc_state);
	free(escaped_svc_state_type);
	free(escaped_current_svc_attempt);
	free(escaped_plugin_output);
	
	/* add the service log */
	result=xlddb_query(sql_statement);
	
	/* there was an error adding the log... */
	if(result==ERROR){
	  
	  xlddb_free_query_memory();
	  
	  snprintf(buffer,sizeof(buffer)-1,"Error: Could not insert record for service log in table '%s' of database '%s'\n",XLDDB_SERVICELOGS_TABLE,xlddb_database);
	  buffer[sizeof(buffer)-1]='\x0';
	  write_to_logs_and_console(buffer,NSLOG_RUNTIME_ERROR,TRUE);
	  
	  /* return an invalid log number */
	  if(log_id!=NULL)
	    *log_id=-1;
	  
	  /* disconnect from the database */
	  xlddb_disconnect();
	  return ERROR;
	}

	/* pass back the id of the log we just added */
	if(log_id!=NULL){
#ifdef USE_XLDMYSQL
	  *log_id=(int)mysql_insert_id(&xlddb_mysql);
#endif
#ifdef USE_XLDPGSQL
	  snprintf(sql_statement,sizeof(sql_statement)-1,"SELECT servicelog_id FROM %s WHERE oid='%s'",XLDDB_SERVICELOGS_TABLE,PQoidStatus(xlddb_pgres));
	  sql_statement[sizeof(sql_statement)-1]='\x0';
	  xlddb_free_query_memory();
	  result=xlddb_query(sql_statement);
	  if(result==ERROR)
	    *log_id=-1;
	  else{
	    if(PQntuples(xlddb_pgres)<1)
	      *log_id=-1;
	    else
	      *log_id=atoi(PQgetvalue(xlddb_pgres,0,0));
	  }
#endif
	}
	
	xlddb_free_query_memory();
	
	/* disconnect from the database */
	xlddb_disconnect();
#ifdef DEBUG0
	printf("xlddb_save_service_log() end\n");
#endif

	return OK;
}



/******************************************************************/
/**************** LOG DELETION FUNCTIONS **********************/
/******************************************************************/


/* deletes a host log */
int xlddb_delete_host_log(int log_id, int bulk_operation){
  char sql_statement[XLDDB_SQL_LENGTH];
  char buffer[MAX_INPUT_BUFFER];
  int result;
  
#ifdef DEBUG0
  printf("xlddb_delete_host_log() start\n");
#endif
  
  /* connect to the database is necessary */
	if(bulk_operation==FALSE){
	  if(xlddb_connect()==ERROR)
	    return ERROR;
	}
	
	/* delete a specific service log */
	snprintf(sql_statement,sizeof(sql_statement)-1,"DELETE FROM %s WHERE hostlog_id='%d'",XLDDB_HOSTLOGS_TABLE,log_id);
	sql_statement[sizeof(sql_statement)-1]='\x0';
	result=xlddb_query(sql_statement);
	
#ifdef USE_XLDPGSQL
	/* free memory used to store query results */
	PQclear(xlddb_pgres);
#endif
	
	/* there was an error */
	if(result==ERROR){
	  
	  snprintf(buffer,sizeof(buffer)-1,"Error: Could not delete host log #%d from table '%s'\n",log_id,XLDDB_HOSTLOGS_TABLE);
	  buffer[sizeof(buffer)-1]='\x0';
	  write_to_logs_and_console(buffer,NSLOG_RUNTIME_WARNING,TRUE);
	}
	
	/* disconnect from the database is necessary */
	if(bulk_operation==FALSE)
	  xlddb_disconnect();
	
#ifdef DEBUG0
	printf("xlddb_delete_host_log() end\n");
#endif
	
	return OK;
}


/* deletes a service log */
int xlddb_delete_service_log(int log_id, int bulk_operation){
  char sql_statement[XLDDB_SQL_LENGTH];
  char buffer[MAX_INPUT_BUFFER];
  int result;
  
#ifdef DEBUG0
  printf("xlddb_delete_service_log() start\n");
#endif
  
  /* connect to the database is necessary */
  if(bulk_operation==FALSE){
    if(xlddb_connect()==ERROR)
      return ERROR;
  }
  
  /* delete a specific service log */
  snprintf(sql_statement,sizeof(sql_statement)-1,"DELETE FROM %s WHERE servicelog_id='%d'",XLDDB_SERVICELOGS_TABLE,log_id);
  sql_statement[sizeof(sql_statement)-1]='\x0';
  result=xlddb_query(sql_statement);
  
#ifdef USE_XLDPGSQL
  /* free memory used to store query results */
  PQclear(xlddb_pgres);
#endif
  
  /* there was an error deleting the log... */
  if(result==ERROR){
    
    snprintf(buffer,sizeof(buffer)-1,"Error: Could not delete service log #%d from table '%s'\n",log_id,XLDDB_SERVICELOGS_TABLE);
    buffer[sizeof(buffer)-1]='\x0';
    write_to_logs_and_console(buffer,NSLOG_RUNTIME_WARNING,TRUE);
  }
  
  /* disconnect from the database is necessary */
  if(bulk_operation==FALSE)
    xlddb_disconnect();
  
#ifdef DEBUG0
  printf("xlddb_delete_service_log() end\n");
#endif
  
  return OK;
}


/* deletes all logs for a particular host */
int xlddb_delete_all_host_logs(char *host_name){
  char sql_statement[XLDDB_SQL_LENGTH];
  char buffer[MAX_INPUT_BUFFER];
  int result;
  
  /* connect to the database */
  if(xlddb_connect()==ERROR)
    return ERROR;
  
  /* delete all logs for the specified host */
  snprintf(sql_statement,sizeof(sql_statement)-1,"DELETE FROM %s WHERE host_name='%s'",XLDDB_HOSTLOGS_TABLE,host_name);
  sql_statement[sizeof(sql_statement)-1]='\x0';
  result=xlddb_query(sql_statement);
  
#ifdef USE_XLDPGSQL
  /* free memory used to store query results */
  PQclear(xlddb_pgres);
#endif
  
  /* there was an error deleting the logs... */
  if(result==ERROR){
    
    snprintf(buffer,sizeof(buffer)-1,"Error: Could not delete all logs for host '%s' from table '%s'\n",host_name,XLDDB_HOSTLOGS_TABLE);
    buffer[sizeof(buffer)-1]='\x0';
    write_to_logs_and_console(buffer,NSLOG_RUNTIME_WARNING,TRUE);
  }
  
  /* disconnect from database */
  xlddb_disconnect();
  
  return OK;
}


/* deletes all logs for a particular service */
int xlddb_delete_all_service_logs(char *host_name, char *svc_description){
  char sql_statement[XLDDB_SQL_LENGTH];
  char buffer[MAX_INPUT_BUFFER];
  int result;
  
  /* connect to the database */
  if(xlddb_connect()==ERROR)
    return ERROR;

  /* delete all logs for the specified service */
  snprintf(sql_statement,sizeof(sql_statement)-1,"DELETE FROM %s WHERE host_name='%s' AND service_description='%s'",XLDDB_SERVICELOGS_TABLE,host_name,svc_description);
  sql_statement[sizeof(sql_statement)-1]='\x0';
  result=xlddb_query(sql_statement);
  
#ifdef USE_XLDPGSQL
  /* free memory used to store query results */
  PQclear(xlddb_pgres);
#endif
  
  /* there was an error deleting the logs... */
  if(result==ERROR){
    
    snprintf(buffer,sizeof(buffer)-1,"Error: Could not delete all logs for service '%s' on host '%s' from table '%s'\n",svc_description,host_name,XLDDB_HOSTLOGS_TABLE);
    buffer[sizeof(buffer)-1]='\x0';
    write_to_logs_and_console(buffer,NSLOG_RUNTIME_WARNING,TRUE);
  }
  
  /* disconnect from database */
  xlddb_disconnect();
  
  return OK;
}

#endif





/* #ifdef NSCGI */

/******************************************************************/
/****************** LOG INPUT FUNCTIONS ***********************/
/******************************************************************/


/* /\* read the log file *\/ */
/* int xlddb_read_log_data(char *config_file){ */

/* 	/\* grab configuration information *\/ */
/* 	if(xlddb_grab_config_info(config_file)==ERROR) */
/* 		return ERROR; */

/* 	/\* initialize database structures *\/ */
/* 	if(xlddb_initialize()==ERROR) */
/* 		return ERROR; */

/* 	/\* establish a connection to the database server *\/ */
/* 	if(xlddb_connect()==ERROR) */
/* 		return ERROR; */

/* 	/\* read in host logs... *\/ */
/* 	xlddb_read_host_logs(); */

/* 	/\* read in service logs... *\/ */
/* 	xlddb_read_service_logs(); */

/* 	/\* close database connection *\/ */
/* 	xlddb_disconnect(); */

/* 	return OK; */
/*         } */


/* /\* reads all host logs *\/ */
/* int xlddb_read_host_logs(void){ */
/* 	char sql_statement[XLDDB_SQL_LENGTH]; */
/* 	int x; */
/* 	time_t entry_time; */
/* 	int persistent; */
/* 	int hostlog_id; */
/* #ifdef USE_XLDMYSQL */
/* 	MYSQL_RES *query_result=NULL; */
/* 	MYSQL_ROW result_row; */
/* #endif */
/* #ifdef USE_XLDPGSQL */
/* 	int ntuples; */
/* 	int tuple; */
/* #endif */


/* 	/\* construct the SQL statement *\/ */
/* #ifdef USE_XLDMYSQL */
/* 	snprintf(sql_statement,sizeof(sql_statement)-1,"SELECT hostlog_id, host_name, UNIX_TIMESTAMP(entry_time) AS entry_time, persistent, author_name, log_data FROM %s",XLDDB_HOSTLOGS_TABLE); */
/* #endif */
/* #ifdef USE_XLDPGSQL */
/* 	snprintf(sql_statement,sizeof(sql_statement)-1,"SELECT hostlog_id, host_name, date_part('epoch',entry_time), persistent, author_name, log_data FROM %s",XLDDB_HOSTLOGS_TABLE); */
/* #endif */
/* 	sql_statement[sizeof(sql_statement)-1]='\x0'; */

/* 	/\* run the query... *\/ */
/* 	if(xlddb_query(sql_statement)==ERROR) */
/* 		return ERROR; */


/* #ifdef USE_XLDMYSQL */
/* 	/\* get the results of the query *\/ */
/* 	if((query_result=mysql_store_result(&xlddb_mysql))==NULL) */
/* 		return ERROR; */

/* 	/\* fetch all rows... *\/ */
/* 	while((result_row=mysql_fetch_row(query_result))!=NULL){ */

/* 		/\* we have six rows - make sure they're not NULL *\/ */
/* 		for(x=0;x<6;x++){ */
/* 			if(result_row[x]==NULL){ */
/* 				mysql_free_result(query_result); */
/* 				return ERROR; */
/* 			        } */
/* 	                } */

/* 		/\* get the log id *\/ */
/* 		hostlog_id=atoi(result_row[0]); */

/* 		/\* entry time *\/ */
/* 		entry_time=(time_t)strtoul(result_row[2],NULL,10); */

/* 		/\* persistent flag *\/ */
/* 		persistent=atoi(result_row[3]); */

/* 		/\* add the host log *\/ */
/* 		add_host_log(result_row[1],entry_time,result_row[4],result_row[5],hostlog_id,persistent); */
/* 	        } */

/* 	/\* free memory used to store query results *\/ */
/* 	mysql_free_result(query_result); */
/* #endif */

/* #ifdef USE_XLDPGSQL */

/* 	/\* how many rows do we have? *\/ */
/* 	ntuples=PQntuples(xlddb_pgres); */

/* 	/\* for each tuple... *\/ */
/* 	for(tuple=0;tuple<ntuples;tuple++){ */

/* 		/\* log id *\/ */
/* 		hostlog_id=atoi(PQgetvalue(xlddb_pgres,tuple,0)); */
		
/* 		/\* entry time *\/ */
/* 		entry_time=(time_t)strtoul(PQgetvalue(xlddb_pgres,tuple,2),NULL,10); */
		
/* 		/\* persistent flag *\/ */
/* 		persistent=atoi(PQgetvalue(xlddb_pgres,tuple,3)); */

/* 		/\* add the host log *\/ */
/* 		add_host_log(PQgetvalue(xlddb_pgres,tuple,1),entry_time,PQgetvalue(xlddb_pgres,tuple,4),PQgetvalue(xlddb_pgres,tuple,5),hostlog_id,persistent); */
/* 	        } */

/* 	/\* free memory used to store query results *\/ */
/* 	PQclear(xlddb_pgres); */
/* #endif */

/* 	return OK; */
/*         } */


/* /\* reads all service logs *\/ */
/* int xlddb_read_service_logs(void){ */
/* 	char sql_statement[XLDDB_SQL_LENGTH]; */
/* 	int x; */
/* 	time_t entry_time; */
/* 	int persistent; */
/* 	int servicelog_id; */
/* #ifdef USE_XLDMYSQL */
/* 	MYSQL_RES *query_result=NULL; */
/* 	MYSQL_ROW result_row; */
/* #endif */
/* #ifdef USE_XLDPGSQL */
/* 	int ntuples; */
/* 	int tuple; */
/* #endif */


/* 	/\* construct the SQL statement *\/ */
/* #ifdef USE_XLDMYSQL */
/* 	snprintf(sql_statement,sizeof(sql_statement)-1,"SELECT servicelog_id, host_name, service_description, UNIX_TIMESTAMP(entry_time) AS entry_time, persistent, author_name, log_data FROM %s",XLDDB_SERVICELOGS_TABLE); */
/* #endif */
/* #ifdef USE_XLDPGSQL */
/* 	snprintf(sql_statement,sizeof(sql_statement)-1,"SELECT servicelog_id, host_name, service_description, date_part('epoch',entry_time), persistent, author_name, log_data FROM %s",XLDDB_SERVICELOGS_TABLE); */
/* #endif */
/* 	sql_statement[sizeof(sql_statement)-1]='\x0'; */

/* 	/\* run the query... *\/ */
/* 	if(xlddb_query(sql_statement)==ERROR) */
/* 		return ERROR; */


/* #ifdef USE_XLDMYSQL */
/* 	/\* get the results of the query *\/ */
/* 	if((query_result=mysql_store_result(&xlddb_mysql))==NULL) */
/* 		return ERROR; */

/* 	/\* fetch all rows... *\/ */
/* 	while((result_row=mysql_fetch_row(query_result))!=NULL){ */

/* 		/\* we have six rows - make sure they're not NULL *\/ */
/* 		for(x=0;x<7;x++){ */
/* 			if(result_row[x]==NULL){ */
/* 				mysql_free_result(query_result); */
/* 				return ERROR; */
/* 			        } */
/* 	                } */

/* 		/\* get the log id *\/ */
/* 		servicelog_id=atoi(result_row[0]); */

/* 		/\* entry time *\/ */
/* 		entry_time=(time_t)strtoul(result_row[3],NULL,10); */

/* 		/\* persistent flag *\/ */
/* 		persistent=atoi(result_row[4]); */

/* 		/\* add the service log *\/ */
/* 		add_service_log(result_row[1],result_row[2],entry_time,result_row[5],result_row[6],servicelog_id,persistent); */
/* 	        } */

/* 	/\* free memory used to store query results *\/ */
/* 	mysql_free_result(query_result); */
/* #endif */

/* #ifdef USE_XLDPGSQL */

/* 	/\* how many rows do we have? *\/ */
/* 	ntuples=PQntuples(xlddb_pgres); */

/* 	/\* for each tuple... *\/ */
/* 	for(tuple=0;tuple<ntuples;tuple++){ */

/* 		/\* log id *\/ */
/* 		servicelog_id=atoi(PQgetvalue(xlddb_pgres,tuple,0)); */
		
/* 		/\* entry time *\/ */
/* 		entry_time=(time_t)strtoul(PQgetvalue(xlddb_pgres,tuple,3),NULL,10); */
		
/* 		/\* persistent flag *\/ */
/* 		persistent=atoi(PQgetvalue(xlddb_pgres,tuple,4)); */

/* 		/\* add the service log *\/ */
/* 		add_service_log(PQgetvalue(xlddb_pgres,tuple,1),PQgetvalue(xlddb_pgres,tuple,2),entry_time,PQgetvalue(xlddb_pgres,tuple,5),PQgetvalue(xlddb_pgres,tuple,6),servicelog_id,persistent); */
/* 	        } */

/* 	/\* free memory used to store query results *\/ */
/* 	PQclear(xlddb_pgres); */
/* #endif */

/* 	return OK; */
/*         } */

/* #endif */



/******************************************************************/
/*********************** MISC FUNCTIONS ***************************/
/******************************************************************/

#ifdef NSCORE

/* escape a string */
int xlddb_escape_string(char *escaped_string, char *raw_string){
	int x,y;

	if(raw_string==NULL || escaped_string==NULL){
		escaped_string=NULL;
		return ERROR;
	        }	  

	/* escape characters */
	for(x=0,y=0;x<strlen(raw_string);x++){
#ifdef USE_XLDMYSQL
		if(raw_string[x]=='\'' || raw_string[x]=='\"' || raw_string[x]=='*' || raw_string[x]=='\\' || raw_string[x]=='$' || raw_string[x]=='?' || raw_string[x]=='.' || raw_string[x]=='^' || raw_string[x]=='+' || raw_string[x]=='[' || raw_string[x]==']' || raw_string[x]=='(' || raw_string[x]==')')
		  escaped_string[y++]='\\';
#endif
#ifdef USE_XLDPGSQL
		if(! (isspace(raw_string[x]) || isalnum(raw_string[x]) || (raw_string[x]=='_')) )
			escaped_string[y++]='\\';
#endif
		escaped_string[y++]=raw_string[x];
	        }
	escaped_string[y]='\0';

	return OK;
	}

#endif

